/* 
 * The Biomechanical ToolKit
 * Copyright (c) 2009-2013, Arnaud Barré
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *     * Redistributions of source code must retain the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer.
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials
 *       provided with the distribution.
 *     * Neither the name(s) of the copyright holders nor the names
 *       of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written
 *       permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "btkMEXObjectHandle.h"
#include "btkMXPoint.h"

#include <btkAcquisition.h>

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  if(nrhs != 2)
    mexErrMsgTxt("Two inputs required.");
  if (nlhs > 3)
    mexErrMsgTxt("Too many output arguments.");

  btk::Acquisition::Pointer acq = btk_MOH_get_object<btk::Acquisition>(prhs[0]);
  
  btk::Point::Pointer point = btkMXGetPoint(acq, nrhs, prhs);
  // Values
  plhs[0] = mxCreateDoubleMatrix(acq->GetPointFrameNumber(), 3, mxREAL);
  memcpy(mxGetPr(plhs[0]), point->GetValues().data(), mxGetNumberOfElements(plhs[0]) * sizeof(double));
  // Residuals
  if (nlhs > 1)
  {
    plhs[1] = mxCreateDoubleMatrix(acq->GetPointFrameNumber(), 1, mxREAL);
    memcpy(mxGetPr(plhs[1]), point->GetResiduals().data(), mxGetNumberOfElements(plhs[1]) * sizeof(double));
  }
  if (nlhs > 2)
  {
    const char* info[] = {"label", "description", "frequency", "units"};
    int numberOfFields =  sizeof(info) / sizeof(char*);
    std::string unit = "";
    if (point->GetType() <= 5) // 0-5: known units.
      unit = acq->GetPointUnit(point->GetType());
    plhs[2] = mxCreateStructMatrix(1, 1, numberOfFields, info);
    mxSetFieldByNumber(plhs[2], 0, 0, mxCreateString(point->GetLabel().c_str()));
    mxSetFieldByNumber(plhs[2], 0, 1, mxCreateString(point->GetDescription().c_str()));
    mxSetFieldByNumber(plhs[2], 0, 2, mxCreateDoubleScalar(acq->GetPointFrequency()));
    mxSetFieldByNumber(plhs[2], 0, 3, mxCreateString(unit.c_str()));
  }
};
